const gulp = require('gulp');
const sass = require('gulp-sass');
const autoprefixer = require('gulp-autoprefixer');
const babel = require('gulp-babel');
const sourcemaps = require('gulp-sourcemaps');

let tasks = [
    { type: 'sass', watch: ['scss/carousel.scss'], source: 'scss/carousel.scss' },
    { type: 'sass', watch: ['scss/flex-content.scss'], source: 'scss/flex-content.scss' },
    { type: 'sass', watch: ['scss/image.scss'], source: 'scss/image.scss' },
    { type: 'sass', watch: ['scss/inline-element.scss'], source: 'scss/inline-element.scss' },
    { type: 'sass', watch: ['scss/tabs.scss'], source: 'scss/tabs.scss' },
    { type: 'sass', watch: ['scss/accordion.scss'], source: 'scss/accordion.scss' },
    { type: 'javascript', watch: ['src/carousel.js'], source: 'src/carousel.js' },
    { type: 'javascript', watch: ['src/tabs.js'], source: 'src/tabs.js' },
    { type: 'javascript', watch: ['src/accordion.js'], source: 'src/accordion.js' },
], taskTypes = {
    sass: task => () => gulp.src(task.source)
        .pipe(sourcemaps.init())
        .pipe(sass({ outputStyle: 'compressed' }).on('error', sass.logError))
        .pipe(autoprefixer())
        .pipe(sourcemaps.write(''))
        .pipe(gulp.dest('css')),
    javascript: task => () => gulp.src(task.source)
        .pipe(sourcemaps.init())
        .pipe(babel({
            presets: ['@babel/env'],
            "targets": {
                "ie": "11"
            }
        }).on('error', console.log))
        .pipe(sourcemaps.write(''))
        .pipe(gulp.dest('js')),
};

tasks.forEach(task => {
    if (taskTypes[task.type]) {
        return gulp.task(task.source, taskTypes[task.type](task));
    } else {
        throw new Error(`No task type for ${task.type} defined!`);
    }
});

gulp.task('make', gulp.parallel(...tasks.map(task => task.source)));
gulp.task('watch-for-changes', () => {
    tasks.map(task => {
        gulp.watch(task.watch, gulp.series(task.source));
    });
});
gulp.task('watch', gulp.series('make', 'watch-for-changes'));
